/***************************************************************************

Fast Freddie memory map (preliminary)

This game runs on a modified Galaxian board. Three color planes, 4 banks
of characters and double the number of sprites were added.

Main CPU

0000-7fff ROM
c000-c7ff RAM
d000-d3ff Video RAM
d400-d7ff Mirrored Video RAM
d800-dbff Color RAM

I/O read:

c800-cfff Custom I/O (Protection)
e000 Input Port 1
e800 Input Port 2
f000 Dip Switches
f800 Watchdog Reset

I/O write:

c800-cfff Custom I/O write? A couple of these locations are
          written to during start up
e000 ?
f001 NMI Enable
f002-f003 Palette Select (?)
f003-f004 Character Bank Select
f006-f007 ?
f116 Screen Flip X
f117 Screen Flip Y
f800 Sound Command Write


Sound CPU

0000-1fff ROM
2000-23ff RAM

I/O read:

3000 Sound Command Read

I/O write:

3000 NMI Enable
4000 ? (Set to 0 at the begining)
5000 AY8910 #1 Control Port
5001 AY8910 #1 Write Port
6000 AY8910 #2 Control Port
6001 AY8910 #2 Write Port

***************************************************************************/

#include "driver.h"
#include "vidhrdw/generic.h"

extern unsigned char *galaxian_attributesram;
void galaxian_attributes_w(int offset,int data);

void fastfred_vh_convert_color_prom(unsigned char *palette, unsigned char *colortable,const unsigned char *color_prom);
void fastfred_vh_screenrefresh(struct osd_bitmap *bitmap);
void fastfred_character_bank_select_w (int offset, int data);
void fastfred_flipx_w(int offset,int data);
void fastfred_flipy_w(int offset,int data);

// This routine is a big hack, but the only way I can get the game working
// without knowing anything about the way the protection chip works.
// These values were derived based on disassembly of the code. Usually, it
// was pretty obvious what the values should be. Of course, this will have
// to change if a different ROM set ever surfaces.
static int custom_io_r(int offset)
{
    switch (cpu_getpc())
    {
    case 0x03c0:
        return 0x9d;

    case 0x03e6:
        return 0x9f;

    case 0x0407:
        return 0x00;

    case 0x0446:
        return 0x94;

    case 0x049f:
        return 0x01;
    case 0x04b1:
        return 0x00;

    case 0x0dd2:
        return 0x00;
    case 0x0de4:
        return 0x20;

    case 0x122b:
        return 0x10;
    case 0x123d:
        return 0x00;

    case 0x1a83:
        return 0x10;
    case 0x1a93:
        return 0x00;

    case 0x1b26:
        return 0x00;
    case 0x1b37:
        return 0x80;

    case 0x2491:
        return 0x10;
    case 0x24a2:
        return 0x00;

    case 0x46ce:
        return 0x20;
    case 0x46df:
        return 0x00;

    case 0x7b18:
        return 0x01;
    case 0x7b29:
        return 0x00;

    case 0x7b47:
        return 0x00;
    case 0x7b58:
        return 0x20;

    }

    if (errorlog) fprintf(errorlog, "Uncaught custom I/O read %04X at %04X\n", 0xc800+offset, cpu_getpc());
    return 0x00;
}


static struct MemoryReadAddress readmem[] =
{
	{ 0x0000, 0x7fff, MRA_ROM },
	{ 0x8000, 0x9fff, MRA_NOP }, // There is a bug in the code that causes
				     // these locations to be read. See 1b5a
				     // One of the instructions should be ld de,
				     // instead of ld hl,
	{ 0xc000, 0xc7ff, MRA_RAM },
	{ 0xc800, 0xcfff, custom_io_r },
	{ 0xd000, 0xd3ff, MRA_RAM },
	{ 0xd800, 0xd8ff, MRA_RAM },
	{ 0xe000, 0xe000, input_port_0_r },
	{ 0xe800, 0xe800, input_port_1_r },
	{ 0xf000, 0xf000, input_port_2_r },
	{ 0xf800, 0xf800, watchdog_reset_r },
	{ -1 }  /* end of table */
};

static struct MemoryReadAddress flyboy_readmem[] =
{
	{ 0x0000, 0x7fff, MRA_ROM },
	{ 0x8000, 0x9fff, MRA_NOP }, // There is a bug in the code that causes
				     // these locations to be read. See 1b5a
				     // One of the instructions should be ld de,
				     // instead of ld hl,
	{ 0xc000, 0xc7ff, MRA_RAM },
	{ 0xc800, 0xcfff, MRA_NOP }, /* This version of Fly-Boy is hacked to not use the protection chip */
	{ 0xd000, 0xd3ff, MRA_RAM },
	{ 0xd800, 0xd8ff, MRA_RAM },
	{ 0xe000, 0xe000, input_port_0_r },
	{ 0xe800, 0xe800, input_port_1_r },
	{ 0xf000, 0xf000, input_port_2_r },
	{ 0xf800, 0xf800, watchdog_reset_r },
	{ -1 }  /* end of table */
};

static struct MemoryWriteAddress writemem[] =
{
	{ 0x0000, 0x7fff, MWA_ROM },
	{ 0xc000, 0xc7ff, MWA_RAM },
	{ 0xc800, 0xcfff, MWA_NOP },
	{ 0xd000, 0xd3ff, videoram_w, &videoram, &videoram_size },
	{ 0xd400, 0xd7ff, videoram_w },  // Mirrored for above
	{ 0xd800, 0xd83f, galaxian_attributes_w, &galaxian_attributesram },
	{ 0xd840, 0xd85f, MWA_RAM, &spriteram, &spriteram_size },
	{ 0xd860, 0xdbff, MWA_NOP }, // Unused, but initialized
	{ 0xe000, 0xe000, MWA_RAM }, // ?
	{ 0xf000, 0xf000, MWA_NOP }, // Unused, but initialized
	{ 0xf001, 0xf001, interrupt_enable_w },
	{ 0xf002, 0xf003, MWA_NOP }, // Unknown (Palette select?)
	{ 0xf004, 0xf005, fastfred_character_bank_select_w },
	{ 0xf006, 0xf007, MWA_RAM }, // ?
	{ 0xf116, 0xf116, fastfred_flipx_w },
	{ 0xf117, 0xf117, fastfred_flipy_w },
	{ 0xf800, 0xf800, soundlatch_w },
	{ -1 }  /* end of table */
};


static struct MemoryReadAddress sound_readmem[] =
{
	{ 0x0000, 0x1fff, MRA_ROM },
	{ 0x2000, 0x23ff, MRA_RAM },
	{ 0x3000, 0x3000, soundlatch_r },
	{ -1 }  /* end of table */
};


static struct MemoryWriteAddress sound_writemem[] =
{
	{ 0x0000, 0x1fff, MWA_ROM },
	{ 0x2000, 0x23ff, MWA_RAM },
	{ 0x3000, 0x3000, interrupt_enable_w },
	{ 0x4000, 0x4000, MWA_RAM },  // ?
	{ 0x5000, 0x5000, AY8910_control_port_0_w },
	{ 0x5001, 0x5001, AY8910_write_port_0_w },
	{ 0x6000, 0x6000, AY8910_control_port_1_w },
	{ 0x6001, 0x6001, AY8910_write_port_1_w },
	{ -1 }  /* end of table */
};


INPUT_PORTS_START( input_ports )
	PORT_START      /* IN1 */
	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_COIN1 )
	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_COIN2 )
	PORT_BITX(    0x04, 0x00, IPT_DIPSWITCH_NAME | IPF_TOGGLE, "Service Mode", OSD_KEY_F2, IP_JOY_NONE, 0 )
	PORT_DIPSETTING(    0x00, "Off" )
	PORT_DIPSETTING(    0x04, "On" )
	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_START1 )
	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_START2 )
	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_COIN3 )
	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_BUTTON1 | IPF_8WAY )
	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_BUTTON1 | IPF_8WAY | IPF_COCKTAIL )

	PORT_START      /* IN2 */
	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_JOYSTICK_LEFT | IPF_8WAY )
	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_JOYSTICK_RIGHT | IPF_8WAY )
	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_JOYSTICK_LEFT | IPF_8WAY | IPF_COCKTAIL )
	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_JOYSTICK_RIGHT | IPF_8WAY | IPF_COCKTAIL )
	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_JOYSTICK_UP | IPF_8WAY )
	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_JOYSTICK_DOWN | IPF_8WAY )
	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_JOYSTICK_UP | IPF_8WAY | IPF_COCKTAIL )
	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_JOYSTICK_DOWN | IPF_8WAY | IPF_COCKTAIL )

	PORT_START      /* DSW 1 */
	PORT_DIPNAME( 0x0f, 0x00, "Coinage", IP_KEY_NONE )
	PORT_DIPSETTING(    0x01, "A 2/1 B 2/1" )
	PORT_DIPSETTING(    0x02, "A 2/1 B 1/3" )
	PORT_DIPSETTING(    0x00, "A 1/1 B 1/1" )
	PORT_DIPSETTING(    0x03, "A 1/1 B 1/2" )
	PORT_DIPSETTING(    0x04, "A 1/1 B 1/3" )
	PORT_DIPSETTING(    0x05, "A 1/1 B 1/4" )
	PORT_DIPSETTING(    0x06, "A 1/1 B 1/5" )
	PORT_DIPSETTING(    0x07, "A 1/1 B 1/6" )
	PORT_DIPSETTING(    0x08, "A 1/2 B 1/2" )
	PORT_DIPSETTING(    0x09, "A 1/2 B 1/4" )
	PORT_DIPSETTING(    0x0a, "A 1/2 B 1/5" )
	PORT_DIPSETTING(    0x0e, "A 1/2 B 1/6" )
	PORT_DIPSETTING(    0x0b, "A 1/2 B 1/10" )
	PORT_DIPSETTING(    0x0c, "A 1/2 B 1/11" )
	PORT_DIPSETTING(    0x0d, "A 1/2 B 1/12" )
	PORT_DIPSETTING(    0x0f, "Free Play" )
	PORT_DIPNAME( 0x10, 0x00, "Lives", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "3" )
	PORT_DIPSETTING(    0x10, "5" )
	PORT_DIPNAME( 0x60, 0x20, "Bonus Life", IP_KEY_NONE )
	PORT_DIPSETTING(    0x20, "30000" )
	PORT_DIPSETTING(    0x40, "50000" )
	PORT_DIPSETTING(    0x60, "100000" )
	PORT_DIPSETTING(    0x00, "None" )
	PORT_DIPNAME( 0x80, 0x00, "Cabinet", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "Upright" )
	PORT_DIPSETTING(    0x80, "Cocktail" )
INPUT_PORTS_END

INPUT_PORTS_START( flyboy_input_ports )
	PORT_START      /* IN1 */
	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_COIN1 )
	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_COIN2 )
	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_UNUSED )
	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_START1 )
	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_START2 )
	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_UNUSED )
	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_BUTTON1 | IPF_8WAY )
	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_BUTTON1 | IPF_8WAY | IPF_COCKTAIL )

	PORT_START      /* IN2 */
	PORT_BIT( 0x01, IP_ACTIVE_HIGH, IPT_JOYSTICK_LEFT | IPF_8WAY )
	PORT_BIT( 0x02, IP_ACTIVE_HIGH, IPT_JOYSTICK_RIGHT | IPF_8WAY )
	PORT_BIT( 0x04, IP_ACTIVE_HIGH, IPT_JOYSTICK_LEFT | IPF_8WAY | IPF_COCKTAIL )
	PORT_BIT( 0x08, IP_ACTIVE_HIGH, IPT_JOYSTICK_RIGHT | IPF_8WAY | IPF_COCKTAIL )
	PORT_BIT( 0x10, IP_ACTIVE_HIGH, IPT_JOYSTICK_UP | IPF_8WAY )
	PORT_BIT( 0x20, IP_ACTIVE_HIGH, IPT_JOYSTICK_DOWN | IPF_8WAY )
	PORT_BIT( 0x40, IP_ACTIVE_HIGH, IPT_JOYSTICK_UP | IPF_8WAY | IPF_COCKTAIL )
	PORT_BIT( 0x80, IP_ACTIVE_HIGH, IPT_JOYSTICK_DOWN | IPF_8WAY | IPF_COCKTAIL )

	PORT_START      /* DSW 1 */
	PORT_DIPNAME( 0x03, 0x00, "Coin A", IP_KEY_NONE )
	PORT_DIPSETTING(    0x03, "6 Coins/1 Credit" )
	PORT_DIPSETTING(    0x02, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x00, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x01, "1 Coin/2 Credits" )
	PORT_DIPNAME( 0x0c, 0x00, "Coin B", IP_KEY_NONE )
	PORT_DIPSETTING(    0x0c, "6 Coins/1 Credit" )
	PORT_DIPSETTING(    0x08, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x00, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x04, "1 Coin/2 Credits" )
	PORT_DIPNAME( 0x30, 0x00, "Lives", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "3" )
	PORT_DIPSETTING(    0x10, "5" )
	PORT_DIPSETTING(    0x20, "7" )
	PORT_BITX( 0,       0x30, IPT_DIPSWITCH_SETTING | IPF_CHEAT, "255", IP_KEY_NONE, IP_JOY_NONE, 0 )
	PORT_DIPNAME( 0x40, 0x00, "Invincibility", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "Off" )
	PORT_DIPSETTING(    0x40, "On" )
	PORT_DIPNAME( 0x80, 0x00, "Cabinet", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "Upright" )
	PORT_DIPSETTING(    0x80, "Cocktail" )
INPUT_PORTS_END



static struct GfxLayout charlayout =
{
	8,8,    /* 8*8 characters */
	256,    /* 256 characters */
	3,      /* 3 bits per pixel */
	{ 0, 0x1000*8, 0x2000*8 }, /* the three bitplanes are separated */
	{ 0, 1, 2, 3, 4, 5, 6, 7 },
	{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },
	8*8     /* every char takes 8 consecutive bytes */
};

static struct GfxLayout spritelayout =
{
	16,16,  /* 16*16 sprites */
	128,    /* 128 sprites */
	3,      /* 3 bits per pixel */
	{ 0, 0x1000*8, 0x2000*8 }, /* the three bitplanes are separated */
	{ 0, 1, 2, 3, 4, 5, 6, 7,
	  8*8+0, 8*8+1, 8*8+2, 8*8+3, 8*8+4, 8*8+5, 8*8+6, 8*8+7 },
	{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8,
			16*8, 17*8, 18*8, 19*8, 20*8, 21*8, 22*8, 23*8 },
	32*8     /* every sprite takes 32 consecutive bytes */
};

static struct GfxDecodeInfo gfxdecodeinfo[] =
{
	{ 1, 0x0000, &spritelayout,   0, 32 },
	{ 1, 0x3000, &charlayout,     0, 32 },
	{ 1, 0x3800, &charlayout,     0, 32 },
	{ 1, 0x6000, &charlayout,     0, 32 },
	{ 1, 0x6800, &charlayout,     0, 32 },
	{ -1 } /* end of array */
};


// These are the Jump Coaster color proms. The ones for Fast Freddie need to
// be dumped.
static unsigned char fastfred_color_prom[] =
{
	/* palette red component*/
	0x00,0x00,0x0c,0x0f,0x0a,0x0f,0x0b,0x0b,0x00,0x00,0x0d,0x00,0x0d,0x0f,0x04,0x0a,
	0x00,0x0f,0x05,0x0d,0x0c,0x09,0x09,0x0a,0x00,0x09,0x0b,0x00,0x00,0x00,0x0d,0x09,
	0x00,0x00,0x00,0x0e,0x07,0x0f,0x0e,0x00,0x00,0x0b,0x0d,0x0e,0x0c,0x0b,0x0f,0x00,
	0x00,0x0f,0x0a,0x0d,0x00,0x08,0x0f,0x00,0x00,0x0e,0x09,0x0d,0x07,0x03,0x0f,0x0f,
	0x00,0x07,0x0c,0x0f,0x05,0x0f,0x05,0x05,0x00,0x00,0x0d,0x00,0x0d,0x0f,0x04,0x0a,
	0x00,0x0f,0x05,0x0d,0x0c,0x09,0x09,0x0a,0x00,0x09,0x0b,0x00,0x00,0x00,0x0d,0x09,
	0x00,0x00,0x00,0x0e,0x07,0x0f,0x0e,0x00,0x00,0x0b,0x0d,0x0e,0x0c,0x0b,0x0f,0x00,
	0x00,0x0f,0x0a,0x0d,0x00,0x08,0x0f,0x00,0x00,0x0e,0x09,0x0d,0x07,0x03,0x0f,0x0f,
	0x00,0x00,0x0c,0x0f,0x0a,0x0f,0x0c,0x0f,0x00,0x00,0x0d,0x00,0x0d,0x0f,0x04,0x0a,
	0x00,0x0f,0x05,0x0d,0x0c,0x09,0x09,0x0a,0x00,0x09,0x0b,0x00,0x00,0x00,0x0d,0x09,
	0x00,0x00,0x00,0x0e,0x07,0x0f,0x0e,0x00,0x00,0x0b,0x0d,0x0e,0x0c,0x0b,0x0f,0x00,
	0x00,0x0f,0x0a,0x0d,0x00,0x08,0x0f,0x00,0x00,0x0e,0x09,0x0d,0x07,0x03,0x0f,0x0f,
	0x00,0x00,0x0c,0x0f,0x07,0x0f,0x08,0x0b,0x00,0x00,0x0d,0x00,0x0d,0x0f,0x04,0x0a,
	0x00,0x0f,0x05,0x0d,0x0c,0x09,0x09,0x0a,0x00,0x09,0x0b,0x00,0x00,0x00,0x0d,0x09,
	0x00,0x00,0x00,0x0e,0x07,0x0f,0x0e,0x00,0x00,0x0b,0x0d,0x0e,0x0c,0x0b,0x0f,0x00,
	0x00,0x0f,0x0a,0x0d,0x00,0x08,0x0f,0x00,0x00,0x0e,0x09,0x0d,0x07,0x03,0x0f,0x0f,

	/* palette green component*/
	0x00,0x0d,0x0c,0x00,0x05,0x0f,0x07,0x0b,0x00,0x0b,0x07,0x00,0x0b,0x0f,0x04,0x07,
	0x00,0x0f,0x04,0x0b,0x0c,0x05,0x09,0x04,0x00,0x00,0x04,0x00,0x0f,0x00,0x00,0x06,
	0x00,0x06,0x00,0x0d,0x00,0x0f,0x00,0x0f,0x00,0x09,0x0c,0x0c,0x0f,0x0a,0x0f,0x08,
	0x00,0x00,0x07,0x0b,0x0b,0x00,0x0f,0x00,0x00,0x0c,0x07,0x09,0x05,0x03,0x0f,0x00,
	0x00,0x0d,0x0c,0x05,0x06,0x0f,0x05,0x0c,0x00,0x0b,0x07,0x00,0x0b,0x0f,0x04,0x07,
	0x00,0x0f,0x04,0x0b,0x0c,0x05,0x09,0x04,0x00,0x00,0x04,0x00,0x0f,0x00,0x00,0x06,
	0x00,0x06,0x00,0x0d,0x00,0x0f,0x00,0x0f,0x00,0x09,0x0c,0x0c,0x0f,0x0a,0x0f,0x08,
	0x00,0x00,0x07,0x0b,0x0b,0x00,0x0f,0x00,0x00,0x0c,0x07,0x09,0x05,0x03,0x0f,0x00,
	0x00,0x0f,0x0c,0x00,0x05,0x0f,0x07,0x0b,0x00,0x0b,0x07,0x00,0x0b,0x0f,0x04,0x0f,
	0x00,0x0f,0x04,0x0b,0x0c,0x05,0x09,0x04,0x00,0x00,0x04,0x00,0x0f,0x00,0x00,0x06,
	0x00,0x06,0x00,0x0d,0x00,0x0f,0x00,0x0f,0x00,0x09,0x0c,0x0c,0x0f,0x0a,0x0f,0x08,
	0x00,0x00,0x07,0x0b,0x0b,0x00,0x0f,0x00,0x00,0x0c,0x07,0x09,0x05,0x03,0x0f,0x00,
	0x00,0x0d,0x0c,0x00,0x07,0x0f,0x08,0x0b,0x00,0x0b,0x07,0x00,0x0b,0x0f,0x04,0x07,
	0x00,0x0f,0x04,0x0b,0x0c,0x05,0x09,0x04,0x00,0x00,0x04,0x00,0x0f,0x00,0x00,0x06,
	0x00,0x06,0x00,0x0d,0x00,0x0f,0x00,0x0f,0x00,0x09,0x0c,0x0c,0x0f,0x0a,0x0f,0x08,
	0x00,0x00,0x07,0x0b,0x0b,0x00,0x0f,0x00,0x00,0x0c,0x07,0x09,0x05,0x03,0x0f,0x00,

	/* palette blue component*/
	0x00,0x00,0x0c,0x00,0x00,0x0f,0x00,0x00,0x00,0x00,0x00,0x0c,0x00,0x00,0x04,0x00,
	0x00,0x00,0x03,0x06,0x0f,0x05,0x0c,0x04,0x00,0x00,0x04,0x0f,0x00,0x00,0x00,0x00,
	0x00,0x00,0x09,0x00,0x00,0x0f,0x00,0x0f,0x00,0x00,0x06,0x00,0x00,0x07,0x00,0x08,
	0x00,0x00,0x00,0x00,0x00,0x08,0x0f,0x0f,0x00,0x00,0x00,0x00,0x03,0x0d,0x0f,0x00,
	0x00,0x07,0x0c,0x05,0x0a,0x0f,0x0d,0x0d,0x00,0x00,0x00,0x0c,0x00,0x00,0x04,0x00,
	0x00,0x00,0x03,0x06,0x0f,0x05,0x0c,0x04,0x00,0x00,0x04,0x0f,0x00,0x00,0x00,0x00,
	0x00,0x00,0x09,0x00,0x00,0x0f,0x00,0x0f,0x00,0x00,0x06,0x00,0x00,0x07,0x00,0x08,
	0x00,0x00,0x00,0x00,0x00,0x08,0x0f,0x0f,0x00,0x00,0x00,0x00,0x03,0x03,0x0f,0x00,
	0x00,0x0a,0x0c,0x00,0x08,0x0f,0x0a,0x0d,0x00,0x00,0x00,0x0c,0x00,0x00,0x04,0x00,
	0x00,0x00,0x03,0x06,0x0f,0x05,0x0c,0x04,0x00,0x00,0x04,0x0f,0x00,0x00,0x00,0x00,
	0x00,0x00,0x09,0x00,0x00,0x0f,0x00,0x0f,0x00,0x00,0x06,0x00,0x00,0x07,0x00,0x08,
	0x00,0x00,0x00,0x00,0x00,0x08,0x0f,0x0f,0x00,0x00,0x00,0x00,0x03,0x03,0x0f,0x00,
	0x00,0x00,0x0c,0x00,0x09,0x0f,0x0a,0x0e,0x00,0x00,0x00,0x0c,0x00,0x00,0x04,0x00,
	0x00,0x00,0x03,0x06,0x0f,0x05,0x0c,0x04,0x00,0x00,0x04,0x0f,0x00,0x00,0x00,0x00,
	0x00,0x00,0x09,0x00,0x00,0x0f,0x00,0x0f,0x00,0x00,0x06,0x00,0x00,0x07,0x00,0x08,
	0x00,0x00,0x00,0x00,0x00,0x08,0x0f,0x0f,0x00,0x00,0x00,0x00,0x03,0x03,0x0f,0x00,
};



static struct AY8910interface ay8910_interface =
{
	2,      /* 2 chips */
	14318000/8,     /* ? */
	{ 0x60ff, 0x60ff }, /* ? */
	{ 0 },
	{ 0 },
	{ 0 },
	{ 0 }
};


static struct MachineDriver machine_driver =
{
	/* basic machine hardware */
	{
		{
			CPU_Z80,
			18432000/6,     /* 3.072 Mhz */
			0,
			readmem,writemem,0,0,
			nmi_interrupt,1
		},
		{
			CPU_Z80 | CPU_AUDIO_CPU,
			14318000/8,     /* ? */
			2,
			sound_readmem,sound_writemem,0,0,
			nmi_interrupt,4
		}
	},
	60, 2500,       /* frames per second, vblank duration */
	1,      /* 1 CPU slice per frame - interleaving is forced when a sound command is written */
	0,

	/* video hardware */
	32*8, 32*8, { 0*8, 32*8-1, 2*8, 30*8-1 },
	gfxdecodeinfo,
	256,32*8,
	fastfred_vh_convert_color_prom,

	VIDEO_TYPE_RASTER,
	0,
	generic_vh_start,
	generic_vh_stop,
	fastfred_vh_screenrefresh,

	/* sound hardware */
	0,0,0,0,
	{
		{
			SOUND_AY8910,
			&ay8910_interface
		}
	}
};

static struct MachineDriver flyboy_machine_driver =
{
	/* basic machine hardware */
	{
		{
			CPU_Z80,
			18432000/6,     /* 3.072 Mhz */
			0,
			flyboy_readmem,writemem,0,0,
			nmi_interrupt,1
		},
		{
			CPU_Z80 | CPU_AUDIO_CPU,
			14318000/8,     /* ? */
			2,
			sound_readmem,sound_writemem,0,0,
			nmi_interrupt,4
		}
	},
	60, 2500,       /* frames per second, vblank duration */
	1,      /* 1 CPU slice per frame - interleaving is forced when a sound command is written */
	0,

	/* video hardware */
	32*8, 32*8, { 0*8, 32*8-1, 2*8, 30*8-1 },
	gfxdecodeinfo,
	256,32*8,
	fastfred_vh_convert_color_prom,

	VIDEO_TYPE_RASTER,
	0,
	generic_vh_start,
	generic_vh_stop,
	fastfred_vh_screenrefresh,

	/* sound hardware */
	0,0,0,0,
	{
		{
			SOUND_AY8910,
			&ay8910_interface
		}
	}
};



/***************************************************************************

  Game driver(s)

***************************************************************************/

ROM_START( fastfred_rom )
	ROM_REGION(0x10000)     /* 64k for main CPU */
	ROM_LOAD( "ffr.01", 0x0000, 0x1000, 0x561ac322 )
	ROM_LOAD( "ffr.02", 0x1000, 0x1000, 0x210c7ce0 )
	ROM_LOAD( "ffr.03", 0x2000, 0x1000, 0x16547e04 )
	ROM_LOAD( "ffr.04", 0x3000, 0x1000, 0x1b0ad21e )
	ROM_LOAD( "ffr.05", 0x4000, 0x1000, 0xc634711c )
	ROM_LOAD( "ffr.06", 0x5000, 0x1000, 0xcc2c7440 )
	ROM_LOAD( "ffr.07", 0x6000, 0x1000, 0x5a6e91dc )
	ROM_LOAD( "ffr.08", 0x7000, 0x1000, 0x055d79f9 )

	ROM_REGION(0x9000)      /* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "ffr.11", 0x0000, 0x1000, 0xb1df2c1b )
	ROM_LOAD( "ffr.12", 0x1000, 0x1000, 0xc0d35047 )
	ROM_LOAD( "ffr.13", 0x2000, 0x1000, 0x3c709c1e )
	ROM_LOAD( "ffr.14", 0x3000, 0x1000, 0x1b1a3bea )
	ROM_LOAD( "ffr.15", 0x4000, 0x1000, 0xf9be8e06 )
	ROM_LOAD( "ffr.16", 0x5000, 0x1000, 0x05f9dda1 )
	ROM_LOAD( "ffr.17", 0x6000, 0x1000, 0xfb0370bd )
	ROM_LOAD( "ffr.18", 0x7000, 0x1000, 0xb2e298ae )
	ROM_LOAD( "ffr.19", 0x8000, 0x1000, 0xc2e2913e )

	ROM_REGION(0x10000)     /* 64k for audio CPU */
	ROM_LOAD( "ffr.09", 0x0000, 0x1000, 0xd1843542 )
	ROM_LOAD( "ffr.10", 0x1000, 0x1000, 0x4bf4aae2 )
ROM_END

ROM_START( flyboy_rom )
	ROM_REGION(0x10000)     /* 64k for main CPU */
	ROM_LOAD( "rom1.cpu", 0x0000, 0x1000, 0x1e28c44c )
	ROM_LOAD( "rom2.cpu", 0x1000, 0x1000, 0x6490e9f2 )
	ROM_LOAD( "rom3.cpu", 0x2000, 0x1000, 0x6b0fa4ed )
	ROM_LOAD( "rom4.cpu", 0x3000, 0x1000, 0xb02bcaa1 )
	ROM_LOAD( "rom5.cpu", 0x4000, 0x1000, 0x459743e9 )
	ROM_LOAD( "rom6.cpu", 0x5000, 0x1000, 0xe369d0a5 )
	ROM_LOAD( "rom7.cpu", 0x6000, 0x1000, 0x2b21cdb9 )
	ROM_LOAD( "rom8.cpu", 0x7000, 0x1000, 0x19db6071 )

	ROM_REGION(0x9000)      /* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "rom11.rom", 0x0000, 0x1000, 0xe9b3ac99 )
	ROM_LOAD( "rom12.rom", 0x1000, 0x1000, 0xe6c05164 )
	ROM_LOAD( "rom13.rom", 0x2000, 0x1000, 0xa88a4fea )
	ROM_LOAD( "rom14.rom", 0x3000, 0x1000, 0x1d7c3b0c )
	ROM_LOAD( "rom15.rom", 0x4000, 0x1000, 0x3a59719f )
	ROM_LOAD( "rom16.rom", 0x5000, 0x1000, 0x8d2c22de )
	ROM_LOAD( "rom17.rom", 0x6000, 0x1000, 0x5a610bc7 )
	ROM_LOAD( "rom18.rom", 0x7000, 0x1000, 0xc3b0a188 )
	ROM_LOAD( "rom19.rom", 0x8000, 0x1000, 0x326aeaa4 )

	ROM_REGION(0x10000)     /* 64k for audio CPU */
	ROM_LOAD( "rom9.cpu",  0x0000, 0x1000, 0x5e7deef7 )
	ROM_LOAD( "rom10.cpu", 0x1000, 0x1000, 0xdaf5a731 )
ROM_END



static int hiload(void)
{
	/* get RAM pointer (this game is multiCPU, we can't assume the global */
	/* RAM pointer is pointing to the right place) */
	unsigned char *RAM = Machine->memory_region[0];


	/* check if the hi score table has already been initialized */
	if (memcmp(&RAM[0xc41c],"\x10",1) == 0)
	{
		void *f;


		if ((f = osd_fopen(Machine->gamedrv->name,0,OSD_FILETYPE_HIGHSCORE,0)) != 0)
		{
			osd_fread(f,&RAM[0xc400],0x45);
			osd_fclose(f);
		}

		return 1;
	}
	else return 0;  /* we can't load the hi scores yet */
}



static void hisave(void)
{
	void *f;
	/* get RAM pointer (this game is multiCPU, we can't assume the global */
	/* RAM pointer is pointing to the right place) */
	unsigned char *RAM = Machine->memory_region[0];


	if ((f = osd_fopen(Machine->gamedrv->name,0,OSD_FILETYPE_HIGHSCORE,1)) != 0)
	{
		osd_fwrite(f,&RAM[0xc400],0x45);
		osd_fclose(f);
	}
}

static int flyboy_hiload(void)
{
	/* get RAM pointer (this game is multiCPU, we can't assume the global */
	/* RAM pointer is pointing to the right place) */
	unsigned char *RAM = Machine->memory_region[0];


	/* check if the hi score table has already been initialized */
	if (memcmp(&RAM[0xc41c],"\x50",1) == 0)
	{
		void *f;


		if ((f = osd_fopen(Machine->gamedrv->name,0,OSD_FILETYPE_HIGHSCORE,0)) != 0)
		{
			osd_fread(f,&RAM[0xc400],0x94);
			osd_fclose(f);
		}

		return 1;
	}
	else return 0;  /* we can't load the hi scores yet */
}



static void flyboy_hisave(void)
{
	void *f;
	/* get RAM pointer (this game is multiCPU, we can't assume the global */
	/* RAM pointer is pointing to the right place) */
	unsigned char *RAM = Machine->memory_region[0];


	if ((f = osd_fopen(Machine->gamedrv->name,0,OSD_FILETYPE_HIGHSCORE,1)) != 0)
	{
		osd_fwrite(f,&RAM[0xc400],0x94);
		osd_fclose(f);
	}
}



struct GameDriver fastfred_driver =
{
	"Fast Freddie",
	"fastfred",
	"Zsolt Vasvari",
	&machine_driver,

	fastfred_rom,
	0, 0,
	0,
	0,      /* sound_prom */

	input_ports,

	fastfred_color_prom, 0, 0,
	ORIENTATION_ROTATE_90,

	hiload, hisave
};

struct GameDriver flyboy_driver =
{
	"Fly-Boy (bootleg?)",
	"flyboy",
	"Zsolt Vasvari\nBrad Oliver (additional code)\nMarco Cassili (additional code)",
	&flyboy_machine_driver,

	flyboy_rom,
	0, 0,
	0,
	0,      /* sound_prom */

	flyboy_input_ports,

	fastfred_color_prom, 0, 0,
	ORIENTATION_ROTATE_90,

	flyboy_hiload, flyboy_hisave
};
