/***************************************************************************

PacLand memory map (preliminary)

Ernesto Corvi
ernesto@imagina.com

Main CPU:
0000-1fff Character ram ( video ram ) + colorram
2000-3000 spriteram?
3000-3800 RAM
3800-3801 Background 1 scrolling
3a00-3a01 Background 2 scrolling
3c00-3c00 Bank and ROM Selector
4000-6000 Banked ROMs
6800-6bff Shared RAM with the MCU
8000-ffff ROM

MCU:
0000-0027 Internal registers, timers and ports.
0040-13ff RAM
1000-13ff Shared RAM with the Main CPU
8000-a000 MCU external ROM
c000-cfff Namco Sound ?
d000-d003 Dip Switches/Joysticks
f000-ffff MCU internal ROM

***************************************************************************/

#include "driver.h"
#include "vidhrdw/generic.h"
#include "m6808/m6808.h"

static unsigned char *sharedram1;
static unsigned char *hd_regs;
static void *pacland_timer = 0;

void pacland_scroll0_w(int offset,int data);
void pacland_scroll1_w(int offset,int data);
void pacland_bankswitch_w(int offset,int data);
void pacland_vh_convert_color_prom(unsigned char *palette, unsigned char *colortable,const unsigned char *color_prom);
int pacland_vh_start(void);
void pacland_vh_stop(void);
void pacland_vh_screenrefresh(struct osd_bitmap *bitmap);


static void pacland_init_machine(void)
{
	if (pacland_timer)
	{
		timer_remove(pacland_timer);

		pacland_timer = 0;
	}
}

static void pacland_halt_mcu_w( int offset, int data )
{
	int v = 0;

	if ( offset == 0 )
		v = 1;

	cpu_halt( 1, v );
}

static int sharedram1_r( int offset )
{
	return sharedram1[offset];
}

static void sharedram1_w( int offset, int val )
{
	sharedram1[offset] = val;
}

void pacland_stop_mcu_timer( void )
{
	if ( pacland_timer )
	{
		timer_remove( pacland_timer );
		pacland_timer = 0;
	}
}

void pacland_timer_proc( int param )
{
	pacland_timer = 0;

	/* update current time with compare time */
	hd_regs[0x09] = hd_regs[0x0b];
	hd_regs[0x0a] = hd_regs[0x0c];

	if ( hd_regs[0x08] & 8 )
		cpu_cause_interrupt( 1, M6808_INT_OCI );
}

void pacland_start_mcu_timer( void )
{
	int total_time, current_time;

	current_time = ( hd_regs[0x09] << 8 ) | hd_regs[0x0a];
	total_time = ( hd_regs[0x0b] << 8 ) | hd_regs[0x0c];

	total_time = ( total_time - current_time );

	total_time &= 0xffff;

	if ( pacland_timer )
		timer_remove( pacland_timer );

	pacland_timer = timer_set( ( ( (double)total_time ) / ((double)Machine->drv->cpu[1].cpu_clock ) ), 0, pacland_timer_proc );
}

static int hd_regs_r( int offset )
{
	if ( offset == 0x02 ) // MCU input port 0
		return readinputport( 4 );
	return hd_regs[offset];
}

static void hd_regs_w( int offset, int data )
{
	if ( offset == 0x0c )	// Timer A
		pacland_start_mcu_timer();
	hd_regs[offset] = data;
}

/* Stubs to pass the correct Dip Switch setup to the MCU */
static int dsw_r0( int offset )
{
	/* Hi 4 bits = DSWA Hi 4 bits */
	/* Lo 4 bits = DSWB Hi 4 bits */
	int r = readinputport( 0 );
	r &= 0xf0;
	r |= ( readinputport( 1 ) >> 4 ) & 0x0f;
	return ~r; /* Active Low */
}

static int dsw_r1( int offset )
{
	/* Hi 4 bits = DSWA Lo 4 bits */
	/* Lo 4 bits = DSWB Lo 4 bits */
	int r = ( readinputport( 0 ) & 0x0f ) << 4;
	r |= readinputport( 1 ) & 0x0f;
	return ~r; /* Active Low */
}

static struct MemoryReadAddress readmem[] =
{
	{ 0x0000, 0x1fff, videoram_r },
	{ 0x2000, 0x37ff, MRA_RAM },
	{ 0x4000, 0x5fff, MRA_BANK1 },
	{ 0x6800, 0x6bff, sharedram1_r },
	{ 0x7800, 0x7800, MRA_NOP },	/* ??? */
	{ 0x8000, 0xffff, MRA_ROM },
	{ -1 }	/* end of table */
};

static struct MemoryWriteAddress writemem[] =
{
	{ 0x0000, 0x2000, videoram_w, &videoram, &videoram_size },
	{ 0x2000, 0x37ff, MWA_RAM },
	{ 0x2700, 0x27ff, MWA_RAM, &spriteram, &spriteram_size },
	{ 0x2f00, 0x2fff, MWA_RAM, &spriteram_2 },
	{ 0x3700, 0x37ff, MWA_RAM, &spriteram_3 },
	{ 0x3800, 0x3801, pacland_scroll0_w },
	{ 0x3a00, 0x3a01, pacland_scroll1_w },
	{ 0x3c00, 0x3c00, pacland_bankswitch_w },
	{ 0x4000, 0x5fff, MWA_ROM },
	{ 0x6800, 0x6bff, sharedram1_w, &sharedram1 },
	{ 0x7000, 0x7000, MWA_NOP },	/* ??? */
	{ 0x7800, 0x7800, MWA_NOP },	/* ??? */
	{ 0x8000, 0x8800, pacland_halt_mcu_w },
	{ 0x9800, 0x9800, MWA_NOP },	/* ??? */
	{ 0x8000, 0xffff, MWA_ROM },
	{ -1 }	/* end of table */
};

static struct MemoryReadAddress mcu_readmem[] =
{
	{ 0x0000, 0x0027, hd_regs_r },
	{ 0x0040, 0x013f, MRA_RAM },
	{ 0x1000, 0x13ff, sharedram1_r },
	{ 0x8000, 0x9fff, MRA_ROM },
	{ 0xc000, 0xc800, MRA_RAM }, // namco sound?
	{ 0xd000, 0xd000, dsw_r0 },
	{ 0xd000, 0xd001, dsw_r1 },
	{ 0xd000, 0xd002, input_port_2_r },
	{ 0xd000, 0xd003, input_port_3_r },
	{ 0xf000, 0xffff, MRA_ROM },
	{ -1 }	/* end of table */
};

static struct MemoryWriteAddress mcu_writemem[] =
{
	{ 0x0000, 0x0027, hd_regs_w, &hd_regs },
	{ 0x0040, 0x013f, MWA_RAM },
	{ 0x1000, 0x13ff, sharedram1_w },
	{ 0x2000, 0x2000, MWA_NOP }, // ???? (w)
	{ 0x4000, 0x4000, MWA_NOP }, // ???? (w)
	{ 0x6000, 0x6000, MWA_NOP }, // ???? (w)
	{ 0x8000, 0x9fff, MWA_ROM },
	{ 0xc000, 0xc7ff, MWA_RAM }, // namco sound?
	{ 0xf000, 0xffff, MWA_ROM },
	{ -1 }	/* end of table */
};



INPUT_PORTS_START( input_ports )
	PORT_START      /* DSWA */
	PORT_DIPNAME( 0x03, 0x00, "Coin B", IP_KEY_NONE )
	PORT_DIPSETTING(    0x03, "3 Coins/1 Credit" )
	PORT_DIPSETTING(    0x02, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x00, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x01, "1 Coin/2 Credits" )
	PORT_DIPNAME( 0x04, 0x00, "Demo Sounds", IP_KEY_NONE )
	PORT_DIPSETTING(    0x04, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x18, 0x00, "Coin A", IP_KEY_NONE )
	PORT_DIPSETTING(    0x18, "3 Coins/1 Credit" )
	PORT_DIPSETTING(    0x10, "2 Coins/1 Credit" )
	PORT_DIPSETTING(    0x00, "1 Coin/1 Credit" )
	PORT_DIPSETTING(    0x08, "1 Coin/2 Credits" )
	PORT_DIPNAME( 0x60, 0x00, "Lives", IP_KEY_NONE )
	PORT_DIPSETTING(    0x20, "2" )
	PORT_DIPSETTING(    0x00, "3" )
	PORT_DIPSETTING(    0x40, "4" )
	PORT_DIPSETTING(    0x60, "5" )
	PORT_DIPNAME( 0x80, 0x00, "Test Mode", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "Off" )
	PORT_DIPSETTING(    0x80, "On" )

	PORT_START      /* DSWB */
	PORT_DIPNAME( 0x01, 0x00, "Start Level Select", IP_KEY_NONE )
	PORT_DIPSETTING(    0x01, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x02, 0x00, "Freeze", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "Off" )
	PORT_DIPSETTING(    0x02, "On" )
	PORT_BITX(    0x04, 0x00, IPT_DIPSWITCH_NAME | IPF_CHEAT, "Round Select", IP_KEY_NONE, IP_JOY_NONE, 0 )
	PORT_DIPSETTING(    0x00, "Off" )
	PORT_DIPSETTING(    0x04, "On" )
	PORT_DIPNAME( 0x18, 0x00, "Difficulty", IP_KEY_NONE )
	PORT_DIPSETTING(    0x08, "Easy" )
	PORT_DIPSETTING(    0x00, "Medium" )
	PORT_DIPSETTING(    0x10, "Hard" )
	PORT_DIPSETTING(    0x18, "Hardest" )
	PORT_DIPNAME( 0xe0, 0x00, "Bonus Life", IP_KEY_NONE )
	PORT_DIPSETTING(    0x00, "30K,80K,130K,300K,500K,1M" )
	PORT_DIPSETTING(    0x20, "30K,100K,200K,400K,600K,1M" )
	PORT_DIPSETTING(    0x40, "40K,100K,180K,300K,500K,1M" )
	PORT_DIPSETTING(    0x60, "30K,80K,Every 100K" )
	PORT_DIPSETTING(    0x80, "50K,150K,Every 200K" )
	PORT_DIPSETTING(    0xa0, "30K,80K,150K" )
	PORT_DIPSETTING(    0xc0, "40K,100K,200K" )
	PORT_DIPSETTING(    0xe0, "40K" )

	PORT_START	/* Memory Mapped Port */
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN )
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_JOYSTICK_UP )
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_JOYSTICK_DOWN | IPF_COCKTAIL )
	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_JOYSTICK_UP | IPF_COCKTAIL )
	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_START1 )
	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_START2 )
	PORT_BITX(    0x40, 0x40, IPT_DIPSWITCH_NAME | IPF_TOGGLE, "Service Mode", OSD_KEY_F2, IP_JOY_NONE, 0 )
	PORT_DIPSETTING(    0x40, "Off" )
	PORT_DIPSETTING(    0x00, "On" )
	PORT_DIPNAME( 0x80, 0x80, "Cabinet", IP_KEY_NONE )
	PORT_DIPSETTING(    0x80, "Upright" )
	PORT_DIPSETTING(    0x00, "Cocktail" )

	PORT_START	/* Memory Mapped Port */
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_UNUSED )
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_UNUSED )
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_UNUSED )
	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_UNUSED )
	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT | IPF_COCKTAIL )
	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_COIN3 )
	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_COIN1 )
	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_COIN2 )

	PORT_START	/* MCU Input Port */
	PORT_BIT( 0x01, IP_ACTIVE_LOW, IPT_UNUSED )
	PORT_BIT( 0x02, IP_ACTIVE_LOW, IPT_UNUSED )
	PORT_BIT( 0x04, IP_ACTIVE_LOW, IPT_UNUSED )
	PORT_BIT( 0x08, IP_ACTIVE_LOW, IPT_BUTTON1 )
	PORT_BIT( 0x10, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT )
	PORT_BIT( 0x20, IP_ACTIVE_LOW, IPT_JOYSTICK_RIGHT )
	PORT_BIT( 0x40, IP_ACTIVE_LOW, IPT_BUTTON1 | IPF_COCKTAIL )
	PORT_BIT( 0x80, IP_ACTIVE_LOW, IPT_JOYSTICK_LEFT | IPF_COCKTAIL )
INPUT_PORTS_END



static struct GfxLayout spritelayout =
{
	16,16,       	/* 16*16 sprites */
	256,           	/* 256 sprites */
	4,              /* 4 bits per pixel */
	{ 0, 4, 16384*8, 16384*8+4 },
	{ 0, 1, 2, 3, 8*8, 8*8+1, 8*8+2, 8*8+3,
			16*8+0, 16*8+1, 16*8+2, 16*8+3, 24*8+0, 24*8+1, 24*8+2, 24*8+3 },
	{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8,
			32*8, 33*8, 34*8, 35*8, 36*8, 37*8, 38*8, 39*8 },
	64*8    /* every sprite takes 256 bytes */
};

static struct GfxLayout charlayout =
{
	8,8,	/* 8*8 characters */
	512,	/* 512 characters */
	2,	/* 2 bits per pixel */
	{ 0, 4 },	/* the bitplanes are packed in the same byte */
	{ 8*8, 8*8+1, 8*8+2, 8*8+3, 0, 1, 2, 3 },
	{ 0*8, 1*8, 2*8, 3*8, 4*8, 5*8, 6*8, 7*8 },
	16*8	/* every char takes 16 consecutive bytes */
};

static struct GfxDecodeInfo gfxdecodeinfo[] =
{
	{ 1, 0x00000, &charlayout,              0, 256 },
	{ 1, 0x02000, &charlayout,          256*4, 256 },
	{ 1, 0x10000, &spritelayout,  256*4+256*4, 2*64 },
	{ 1, 0x18000, &spritelayout,  256*4+256*4, 2*64 },
	{ -1 } /* end of array */
};



static unsigned char color_prom[] =
{
	/* PL1-2.BIN : Green,Red */
	0x00,0x11,0xD3,0x6A,0x7C,0x8C,0xC3,0x49,0xB2,0x37,0x73,0x57,0x79,0x68,0x46,0x00,
	0x92,0x41,0x73,0x20,0x8C,0xBB,0x67,0x45,0x00,0xE9,0xA7,0x44,0x88,0xCC,0x35,0x11,
	0xE6,0x00,0xE6,0xC4,0x82,0x0F,0xF0,0xCE,0xAC,0x79,0x35,0xDF,0xAC,0xB4,0x7A,0x38,
	0x00,0x11,0x44,0xE6,0xBD,0x5A,0x36,0x73,0x8C,0xCE,0x47,0x00,0xEE,0x57,0x78,0xBC,
	0x57,0xDF,0x00,0x11,0x66,0xCE,0x72,0x55,0x79,0x9A,0xAB,0xBC,0x57,0xB0,0xC6,0x00,
	0xE6,0xDD,0x44,0xEE,0x55,0x11,0x81,0x91,0xA2,0xD3,0xA2,0x90,0xF0,0x75,0x00,0x58,
	0x8C,0xCC,0x99,0x92,0xAA,0x4A,0x38,0x77,0xAC,0x00,0x00,0xEE,0x26,0x55,0xCE,0x00,
	0x2F,0xEE,0xEF,0x8E,0x83,0xD0,0x9E,0x61,0x79,0x40,0x30,0xAC,0x68,0x4C,0xEE,0x00,
	0xEE,0xAE,0x0E,0x97,0x2E,0xAE,0x7C,0xE0,0x4E,0x4A,0x58,0x66,0x44,0xEE,0x08,0xAA,
	0x00,0xA0,0x83,0x6C,0x9E,0x49,0x5D,0x7F,0x26,0x48,0xCC,0x8A,0x68,0xCC,0xAA,0x17,
	0x0B,0x59,0x7C,0xE4,0x09,0x16,0x00,0xDA,0xFF,0x2E,0x21,0xD9,0x92,0x79,0xCE,0x6D,
	0x3B,0x8A,0xEF,0xCD,0xAB,0x44,0x99,0xE0,0x80,0x6A,0x7A,0xE0,0x0E,0x8B,0xA0,0xAA,
	0x77,0x69,0x55,0x58,0xAE,0x8C,0xCF,0x8F,0x6C,0x0D,0xFA,0x44,0xC6,0x9D,0x7A,0x00,
	0x8C,0x81,0xFF,0xFF,0xEE,0xBF,0xAF,0xFF,0xDF,0xEF,0xCC,0xAA,0x0A,0x08,0x88,0x6A,
	0xEE,0x11,0x48,0x20,0x20,0xCC,0xAA,0x0C,0x9F,0x08,0xFA,0xD8,0x90,0x2E,0x00,0x00,
	0xEE,0x11,0xFF,0x2E,0xAA,0x44,0x99,0x00,0xEE,0x0E,0x66,0x00,0x00,0x00,0x00,0x00,
	0x00,0x11,0x57,0x59,0x6A,0x8C,0x9E,0x47,0x78,0x45,0x68,0x57,0x79,0x68,0x46,0x00,
	0x78,0x56,0x5A,0x26,0x8C,0x79,0x56,0x34,0x04,0xC8,0xA7,0x44,0x88,0xCC,0x35,0x11,
	0x9E,0x11,0x76,0x78,0x75,0x0D,0xA6,0x9D,0x69,0x59,0x35,0xBD,0x9B,0x68,0x57,0x25,
	0x00,0x11,0x44,0x9E,0xBD,0x5A,0x36,0x76,0x8C,0xCE,0x47,0x00,0xBE,0x56,0x67,0xBC,
	0x57,0xDF,0x03,0x11,0x66,0xCE,0x5A,0x55,0x79,0x9A,0xAB,0xBC,0x57,0xB0,0xC6,0x00,
	0x9E,0xCF,0x68,0xAA,0x66,0x11,0x75,0x78,0x7C,0x96,0x69,0x86,0x95,0x57,0x00,0x47,
	0x8C,0xBC,0x8A,0x54,0x8A,0x5A,0x7B,0x39,0x8B,0x36,0x24,0xEE,0x48,0x69,0xCE,0x00,
	0x2F,0xEE,0xEF,0x8E,0x83,0xD0,0x9E,0x61,0x79,0x40,0x30,0xAC,0x22,0x4C,0xEE,0x00,
	0xEE,0xAE,0x0E,0x97,0x2E,0xAE,0x7C,0xE0,0x4E,0x4A,0x58,0x66,0x44,0xEE,0x08,0xAA,
	0x00,0xA0,0x83,0x6C,0x9E,0x48,0x5D,0x7F,0x26,0x48,0xCC,0x8A,0x68,0xCC,0xAA,0x17,
	0x0B,0x59,0x7C,0xE4,0x09,0x16,0x00,0xDA,0xFF,0x2E,0x21,0xD9,0x92,0x79,0xCE,0x6D,
	0x3B,0x8A,0xEF,0xCD,0xAB,0x44,0x99,0xE0,0x80,0x6A,0x7A,0xE0,0x0E,0x8B,0xA0,0xAA,
	0x77,0x69,0x55,0x58,0xAE,0x8C,0xCF,0x8F,0x6C,0x0D,0xFA,0x44,0xC6,0x9D,0x7A,0x00,
	0x8C,0x75,0xFF,0xFF,0xEE,0xBF,0xAF,0xFF,0xDF,0xEF,0xCC,0xAA,0x0A,0x08,0x88,0x6A,
	0xEE,0x11,0x48,0x20,0x20,0xCC,0xAA,0x0C,0x9F,0x08,0xFA,0xD8,0x90,0x2E,0x00,0x00,
	0xEE,0x11,0xFF,0x2E,0xAA,0x44,0x99,0x00,0xEE,0x0E,0x66,0x00,0x00,0x00,0x00,0x00,
	0x00,0x11,0x64,0x35,0x68,0x8C,0x22,0x35,0x64,0x34,0x33,0x57,0x79,0x68,0x46,0x00,
	0x62,0x30,0x42,0x22,0x8C,0x79,0x56,0x34,0x00,0xC8,0xA7,0x44,0x88,0xCC,0x35,0x11,
	0x00,0x11,0x53,0x42,0x63,0x09,0xA0,0x99,0x77,0x65,0x74,0xAA,0x77,0x96,0x66,0x22,
	0x00,0x11,0x44,0x22,0xBD,0x5A,0x36,0x53,0x8C,0xCE,0x47,0x00,0x99,0x56,0x67,0xBC,
	0x56,0xDF,0x00,0x11,0x66,0xCE,0x42,0x55,0x79,0x9A,0xAB,0xBC,0x57,0xB0,0xC6,0x00,
	0x22,0x99,0x66,0xAA,0x66,0x11,0x53,0x43,0x42,0x83,0x63,0x52,0x60,0x75,0x00,0x47,
	0x7B,0x98,0x76,0x42,0x77,0x46,0x77,0x36,0x77,0x33,0x00,0xEE,0x45,0x44,0xCE,0x00,
	0x2F,0xEE,0xEF,0x8E,0x83,0xD0,0x9E,0x61,0x79,0x40,0x30,0xAC,0xA2,0x4C,0xEE,0x00,
	0xEE,0xAE,0x0E,0x97,0x2E,0xAE,0x7C,0xE0,0x4E,0x4A,0x58,0x66,0x44,0xEE,0x08,0xAA,
	0x00,0xA0,0x83,0x6C,0x9E,0x48,0x5D,0x7F,0x26,0x48,0xCC,0x8A,0x68,0xCC,0xAA,0x17,
	0x0B,0x59,0x7C,0xE4,0x09,0x16,0x00,0xDA,0xFF,0x2E,0x21,0xD9,0x92,0x79,0xCE,0x6D,
	0x3B,0x8A,0xEF,0xCD,0xAB,0x44,0x99,0xE0,0x80,0x6A,0x7A,0xE0,0x0E,0x8B,0xA0,0xAA,
	0x77,0x69,0x55,0x58,0xAE,0x8C,0xCF,0x8F,0x6C,0x0D,0xFA,0x44,0xC6,0x9D,0x7A,0x00,
	0x8C,0x53,0xFF,0xFF,0xEE,0xBF,0xAF,0xFF,0xDF,0xEF,0xCC,0xAA,0x0A,0x08,0x88,0x6A,
	0xEE,0x11,0x48,0x20,0x20,0xCC,0xAA,0x0C,0x9F,0x08,0xFA,0xD8,0x90,0x2E,0x00,0x00,
	0xEE,0x11,0xFF,0x2E,0xAA,0x44,0x99,0x00,0xEE,0x0E,0x66,0x00,0x00,0x00,0x00,0x00,
	0x00,0x11,0x95,0x59,0x6A,0x8C,0x41,0x38,0x73,0x36,0x53,0x57,0x79,0x68,0x46,0x00,
	0x62,0x40,0x64,0x43,0x8C,0x79,0x56,0x34,0x00,0xC8,0xA7,0x44,0x88,0xCC,0x35,0x11,
	0x97,0x11,0x83,0xA5,0x73,0x0D,0xD0,0x9C,0x79,0xB8,0x95,0xBB,0x99,0xA6,0x66,0x22,
	0x00,0x11,0x44,0x97,0xBD,0x5A,0x36,0x62,0x8C,0xCE,0x47,0x00,0xAA,0x56,0x67,0xBC,
	0x56,0xDF,0x00,0x11,0x66,0xCE,0x62,0x55,0x79,0x9A,0xAB,0xBC,0x57,0xB0,0xC6,0x00,
	0x97,0xCC,0x66,0xAA,0x66,0x11,0x83,0xA5,0x65,0xA5,0x94,0x80,0xA0,0x75,0x00,0x47,
	0x8C,0xBA,0x98,0x62,0x99,0x58,0x79,0x38,0x8A,0x33,0x00,0xEE,0x47,0x68,0xCE,0x00,
	0x2F,0xEE,0xEF,0x8E,0x83,0xD0,0x9E,0x61,0x79,0x40,0x30,0xAC,0xAD,0x4C,0xEE,0x00,
	0xEE,0xAE,0x0E,0x97,0x2E,0xAE,0x7C,0xE0,0x4E,0x4A,0x58,0x66,0x44,0xEE,0x08,0xAA,
	0x00,0xA0,0x83,0x6C,0x9E,0x48,0x5D,0x7F,0x26,0x48,0xCC,0x8A,0x68,0xCC,0xAA,0x17,
	0x0B,0x59,0x7C,0xE4,0x09,0x16,0x00,0xDA,0xFF,0x2E,0x21,0xD9,0x92,0x79,0xCE,0x6D,
	0x3B,0x8A,0xEF,0xCD,0xAB,0x44,0x99,0xE0,0x80,0x6A,0x7A,0xE0,0x0E,0x8B,0xA0,0xAA,
	0x77,0x69,0x55,0x58,0xAE,0x8C,0xCF,0x8F,0x6C,0x0D,0xFA,0x44,0xC6,0x9D,0x7A,0x00,
	0x8C,0x83,0xFF,0xFF,0xEE,0xBF,0xAF,0xFF,0xDF,0xEF,0xCC,0xAA,0x0A,0x08,0x88,0x6A,
	0xEE,0x11,0x48,0x20,0x20,0xCC,0xAA,0x0C,0x9F,0x08,0xFA,0xD8,0x90,0x2E,0x00,0x00,
	0xEE,0x11,0xFF,0x2E,0xAA,0x44,0x99,0x00,0xEE,0x0E,0x66,0x00,0x00,0x00,0x00,0x00,

	/* PL1-1.BIN : Blue */
	0x00,0x01,0x03,0x04,0x03,0x05,0x0E,0x03,0x02,0x02,0x03,0x03,0x05,0x07,0x02,0x00,
	0x00,0x00,0x04,0x04,0x05,0x07,0x05,0x03,0x0E,0x0F,0x0D,0x04,0x02,0x00,0x01,0x01,
	0x0F,0x00,0x06,0x04,0x02,0x00,0x00,0x0C,0x0A,0x0C,0x08,0x0D,0x0A,0x0F,0x07,0x03,
	0x00,0x01,0x04,0x0F,0x05,0x05,0x03,0x03,0x05,0x08,0x04,0x00,0x0E,0x04,0x05,0x09,
	0x03,0x0C,0x0A,0x01,0x05,0x0A,0x04,0x05,0x05,0x06,0x07,0x08,0x03,0x04,0x0A,0x00,
	0x0F,0x0D,0x04,0x0E,0x05,0x01,0x04,0x05,0x07,0x03,0x02,0x00,0x00,0x0E,0x0F,0x03,
	0x05,0x0C,0x09,0x0D,0x0A,0x04,0x00,0x0E,0x0C,0x0F,0x08,0x00,0x02,0x08,0x00,0x0C,
	0x02,0x0E,0x00,0x00,0x04,0x00,0x09,0x02,0x0C,0x01,0x00,0x0C,0x0B,0x04,0x0E,0x00,
	0x00,0x04,0x00,0x0C,0x02,0x0E,0x08,0x0E,0x04,0x0E,0x08,0x06,0x04,0x00,0x00,0x00,
	0x0E,0x0A,0x0E,0x00,0x08,0x03,0x05,0x07,0x00,0x04,0x08,0x06,0x04,0x0C,0x0A,0x01,
	0x00,0x00,0x08,0x0D,0x00,0x0B,0x0A,0x0F,0x06,0x02,0x0F,0x02,0x02,0x05,0x0A,0x0D,
	0x0B,0x05,0x09,0x07,0x05,0x09,0x0E,0x00,0x00,0x04,0x04,0x0E,0x00,0x0B,0x0A,0x0C,
	0x0A,0x06,0x08,0x01,0x00,0x00,0x08,0x0C,0x0A,0x00,0x0F,0x0F,0x0F,0x06,0x03,0x00,
	0x05,0x04,0x0F,0x00,0x00,0x00,0x00,0x0A,0x05,0x06,0x0C,0x0A,0x00,0x00,0x08,0x00,
	0x0E,0x01,0x00,0x0A,0x06,0x0C,0x00,0x04,0x00,0x02,0x00,0x00,0x00,0x00,0x00,0x00,
	0x0E,0x01,0x06,0x02,0x0A,0x0E,0x0E,0x0E,0x00,0x00,0x0F,0x00,0x00,0x00,0x00,0x00,
	0x00,0x01,0x02,0x02,0x03,0x05,0x04,0x02,0x03,0x02,0x03,0x03,0x05,0x07,0x02,0x00,
	0x01,0x01,0x03,0x03,0x05,0x05,0x04,0x03,0x0A,0x0D,0x0D,0x04,0x02,0x00,0x01,0x01,
	0x04,0x01,0x03,0x03,0x03,0x00,0x00,0x09,0x06,0x0A,0x05,0x0B,0x09,0x0D,0x07,0x07,
	0x00,0x01,0x04,0x04,0x05,0x05,0x03,0x03,0x05,0x08,0x04,0x00,0x0B,0x04,0x04,0x09,
	0x03,0x0C,0x0A,0x01,0x05,0x0A,0x03,0x05,0x05,0x06,0x07,0x08,0x03,0x04,0x0A,0x00,
	0x04,0x0C,0x06,0x0A,0x06,0x01,0x03,0x03,0x04,0x07,0x03,0x02,0x00,0x0E,0x0E,0x03,
	0x05,0x0A,0x07,0x06,0x08,0x05,0x05,0x00,0x06,0x08,0x07,0x00,0x04,0x04,0x00,0x0C,
	0x02,0x0E,0x00,0x00,0x04,0x00,0x09,0x02,0x0C,0x01,0x00,0x0C,0x08,0x04,0x0E,0x00,
	0x00,0x04,0x00,0x0C,0x02,0x0E,0x08,0x0E,0x04,0x0E,0x08,0x06,0x04,0x00,0x00,0x00,
	0x0E,0x0A,0x0E,0x00,0x08,0x03,0x05,0x07,0x00,0x04,0x08,0x06,0x04,0x0C,0x0A,0x01,
	0x00,0x00,0x08,0x0D,0x00,0x0B,0x0A,0x0F,0x06,0x02,0x0F,0x02,0x02,0x05,0x0A,0x0D,
	0x0B,0x05,0x09,0x07,0x05,0x09,0x0E,0x00,0x00,0x04,0x04,0x0E,0x00,0x0B,0x0A,0x0C,
	0x0A,0x06,0x08,0x01,0x00,0x00,0x08,0x0C,0x0A,0x00,0x0F,0x0F,0x0F,0x06,0x03,0x00,
	0x05,0x03,0x0F,0x00,0x00,0x00,0x00,0x0A,0x05,0x06,0x0C,0x0A,0x00,0x00,0x08,0x00,
	0x0E,0x01,0x00,0x0A,0x06,0x0C,0x0A,0x00,0x01,0x00,0x06,0x04,0x00,0x02,0x00,0x00,
	0x0E,0x01,0x06,0x02,0x0A,0x0E,0x0E,0x0E,0x00,0x00,0x0F,0x00,0x00,0x00,0x00,0x00,
	0x00,0x01,0x06,0x03,0x04,0x05,0x08,0x08,0x06,0x07,0x0A,0x03,0x05,0x07,0x02,0x00,
	0x08,0x05,0x07,0x06,0x05,0x05,0x04,0x03,0x0E,0x0D,0x0D,0x04,0x02,0x00,0x01,0x01,
	0x00,0x01,0x06,0x07,0x08,0x07,0x07,0x0C,0x09,0x08,0x07,0x0D,0x0B,0x0C,0x0A,0x09,
	0x00,0x01,0x04,0x08,0x05,0x05,0x03,0x06,0x05,0x08,0x04,0x00,0x0A,0x04,0x04,0x09,
	0x03,0x0C,0x0A,0x01,0x05,0x0A,0x07,0x05,0x05,0x06,0x07,0x08,0x03,0x04,0x0A,0x00,
	0x08,0x0A,0x06,0x0A,0x06,0x01,0x06,0x07,0x07,0x07,0x07,0x07,0x04,0x0E,0x0E,0x03,
	0x05,0x0C,0x09,0x09,0x09,0x07,0x09,0x06,0x08,0x0A,0x09,0x00,0x06,0x06,0x00,0x0C,
	0x02,0x0E,0x00,0x00,0x04,0x00,0x09,0x02,0x0C,0x01,0x00,0x0C,0x02,0x04,0x0E,0x00,
	0x00,0x04,0x00,0x0C,0x02,0x0E,0x08,0x0E,0x04,0x0E,0x08,0x06,0x04,0x00,0x00,0x00,
	0x0E,0x0A,0x0E,0x00,0x08,0x03,0x05,0x07,0x00,0x04,0x08,0x06,0x04,0x0C,0x0A,0x01,
	0x00,0x00,0x08,0x0D,0x00,0x0B,0x0A,0x0F,0x06,0x02,0x0F,0x02,0x02,0x05,0x0A,0x0D,
	0x0B,0x05,0x09,0x07,0x05,0x09,0x0E,0x00,0x00,0x04,0x04,0x0E,0x00,0x0B,0x0A,0x0C,
	0x0A,0x06,0x08,0x01,0x00,0x00,0x08,0x0C,0x0A,0x00,0x0F,0x0F,0x0F,0x06,0x03,0x00,
	0x05,0x06,0x0F,0x00,0x00,0x00,0x00,0x0A,0x05,0x06,0x0C,0x0A,0x00,0x00,0x08,0x00,
	0x0E,0x01,0x00,0x0A,0x06,0x0C,0x0A,0x00,0x01,0x00,0x06,0x04,0x00,0x02,0x00,0x00,
	0x0E,0x01,0x06,0x02,0x0A,0x0E,0x0E,0x0E,0x00,0x00,0x0F,0x00,0x00,0x00,0x00,0x00,
	0x00,0x01,0x05,0x03,0x03,0x05,0x0E,0x02,0x04,0x02,0x03,0x03,0x05,0x07,0x02,0x00,
	0x06,0x04,0x08,0x05,0x05,0x05,0x04,0x03,0x0E,0x0D,0x0D,0x04,0x02,0x00,0x01,0x01,
	0x0A,0x01,0x05,0x07,0x03,0x00,0x00,0x09,0x07,0x09,0x06,0x0D,0x0B,0x0D,0x09,0x07,
	0x00,0x01,0x04,0x0A,0x05,0x05,0x03,0x06,0x05,0x08,0x04,0x00,0x0A,0x04,0x04,0x09,
	0x03,0x0C,0x0A,0x01,0x05,0x0A,0x04,0x05,0x05,0x06,0x07,0x08,0x03,0x04,0x0A,0x00,
	0x0A,0x0F,0x06,0x0A,0x06,0x01,0x05,0x07,0x08,0x07,0x06,0x00,0x00,0x0E,0x0E,0x03,
	0x05,0x0C,0x0A,0x08,0x09,0x05,0x05,0x00,0x06,0x0C,0x0A,0x00,0x04,0x04,0x00,0x0C,
	0x02,0x0E,0x00,0x00,0x04,0x00,0x09,0x02,0x0C,0x01,0x00,0x0C,0x04,0x04,0x0E,0x00,
	0x00,0x04,0x00,0x0C,0x02,0x0E,0x08,0x0E,0x04,0x0E,0x08,0x06,0x04,0x00,0x00,0x00,
	0x0E,0x0A,0x0E,0x00,0x08,0x03,0x05,0x07,0x00,0x04,0x08,0x06,0x04,0x0C,0x0A,0x01,
	0x00,0x00,0x08,0x0D,0x00,0x0B,0x0A,0x0F,0x06,0x02,0x0F,0x02,0x02,0x05,0x0A,0x0D,
	0x0B,0x05,0x09,0x07,0x05,0x09,0x0E,0x00,0x00,0x04,0x04,0x0E,0x00,0x0B,0x0A,0x0C,
	0x0A,0x06,0x08,0x01,0x00,0x00,0x08,0x0C,0x0A,0x00,0x0F,0x0F,0x0F,0x06,0x03,0x00,
	0x05,0x05,0x0F,0x00,0x00,0x00,0x00,0x0A,0x05,0x06,0x0C,0x0A,0x00,0x00,0x08,0x00,
	0x0E,0x01,0x00,0x0A,0x06,0x0C,0x0A,0x00,0x01,0x00,0x06,0x04,0x00,0x02,0x00,0x00,
	0x0E,0x01,0x06,0x02,0x0A,0x0E,0x0E,0x0E,0x00,0x00,0x0F,0x00,0x00,0x00,0x00,0x00,

	/* PL1-3.BIN : Sprites prom */
	0xFF,0xE1,0xF0,0x80,0x82,0x81,0x80,0x80,0xE1,0xFF,0x82,0xE1,0xA4,0x80,0xF1,0x00,
	0xFF,0xE1,0xF0,0x80,0x82,0x81,0xE1,0x82,0xE1,0xFF,0x82,0xE1,0xA4,0x80,0xF1,0x00,
	0xFF,0xE1,0xF0,0x80,0xA9,0x81,0x80,0x80,0xF0,0xA6,0xA6,0xA6,0xA4,0xA6,0xF1,0x00,
	0xFF,0xE1,0xF0,0x80,0xA9,0x81,0xE1,0x82,0xF0,0xA6,0xA6,0xA6,0xA4,0xA6,0xF1,0x00,
	0xFF,0xE1,0x82,0x81,0xFF,0xFF,0xFF,0xE1,0xE1,0x82,0x81,0xE1,0x82,0x81,0x82,0x81,
	0xFF,0xE1,0x82,0x81,0xE1,0x82,0x81,0xFF,0x82,0xFF,0xFF,0x81,0xE1,0xE1,0x81,0x82,
	0xFF,0xE1,0xF0,0x81,0x82,0x8B,0x8C,0x8D,0xA1,0x8F,0x90,0x92,0x91,0x93,0x00,0xF1,
	0xFF,0xE1,0x85,0x81,0xFF,0xFF,0xFF,0xE1,0x85,0x81,0xE1,0xE1,0x85,0x81,0x85,0x81,
	0xFF,0xE1,0x85,0x81,0xE1,0x85,0x81,0xFF,0xFF,0xFF,0x85,0x81,0xE1,0xE1,0x81,0x85,
	0xFF,0xE1,0xF0,0x85,0x81,0x8B,0x8C,0x8D,0xA2,0x8F,0x90,0x92,0x91,0x94,0x95,0xF1,
	0xFF,0xE1,0x87,0x81,0xFF,0xFF,0xFF,0xE1,0x87,0x81,0xE1,0xE1,0x87,0x81,0x87,0x81,
	0xFF,0xE1,0x87,0x81,0xE1,0x87,0x81,0xFF,0xFF,0xFF,0x87,0x81,0xE1,0xE1,0x81,0x87,
	0xFF,0xE1,0xF0,0x87,0x81,0x8B,0x8C,0x8D,0xA3,0x8F,0x90,0x92,0x91,0x00,0x00,0xF1,
	0xFF,0xE1,0x82,0x81,0xFF,0xFF,0xFF,0xE1,0xE1,0x82,0x81,0xE1,0x82,0x81,0x82,0x81,
	0xFF,0xE1,0x82,0x81,0xE1,0x82,0x81,0xFF,0x82,0xFF,0xFF,0x81,0xE1,0xE1,0x81,0x82,
	0xFF,0xE1,0xF0,0xA0,0x81,0x8B,0x8C,0x8D,0xA4,0x8F,0x90,0x92,0x91,0x00,0x00,0xF1,
	0xFF,0xE1,0x89,0x81,0xFF,0xFF,0xFF,0xE1,0x89,0x81,0xE1,0xE1,0x89,0x81,0x89,0x81,
	0xFF,0xE1,0x89,0x81,0xE1,0x89,0x81,0xFF,0xFF,0xFF,0x89,0x81,0xE1,0xE1,0x81,0x89,
	0xFF,0xE1,0xF0,0x89,0x81,0x8B,0x8C,0x8D,0x8A,0x8F,0x90,0x22,0x91,0x8A,0x00,0xF1,
	0xFF,0xFF,0xF9,0xFF,0xFF,0xF9,0xFF,0xFF,0xFF,0xFF,0xFA,0x80,0xFF,0xFF,0xFF,0xFF,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xF1,0xD0,0xD0,0xD0,0xF1,0xF1,0xFF,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xF1,0xF0,0xB7,0x85,0xFF,0x7F,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xE7,0xE9,0xF0,0xE8,0xF0,0xF0,0xF0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xE1,0x8B,0x8C,0x82,0x96,0x8E,0x90,0xF0,0x97,0x81,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xE1,0x98,0x98,0x99,0x9B,0x9C,0x9F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xE0,0x9D,0x9E,0x8B,0x8C,0x90,0xA6,0xA7,0xE1,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xF0,0xF2,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF0,0xF3,0xF0,0xF0,0xF0,0xF0,
	0xFF,0xE1,0xAB,0xAC,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xE1,0xAD,0xAE,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xE1,0xE0,0xAF,0x00,0x00,0x00,0x00,0xB0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xF1,0xF0,0xFF,0xFF,0xFF,0xFF,0xFF,0xF1,0xFF,0xFF,0xF1,0xFF,0xFF,0xF1,0xFF,
	0xFF,0xE0,0xE0,0xE0,0x81,0x85,0x87,0x89,0x82,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xE0,0xE0,0x81,0x87,0x89,0x82,0x85,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xF0,0xF0,0xE0,0x81,0x89,0x82,0x85,0x87,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xF0,0xF0,0xF0,0x81,0x82,0x85,0x87,0x89,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x95,0x98,0x99,0x9A,0x9B,0x9C,0x9F,0xE0,0x9E,0x9D,0xE1,0x00,0x00,0x00,0x00,
	0xF1,0xD1,0xD1,0xD1,0xF1,0xD1,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xD0,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xE1,0xB1,0xB2,0xB3,0xB4,0xE0,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x7F,0xFF,0x7F,0xFF,0x7F,0xFF,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x7F,0xFF,0x7F,0xFF,0xFF,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x7F,0xFF,0xFF,0x7F,0x7F,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x7F,0x7F,0x7F,0x7F,0x7F,0x7F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x90,0xB5,0xF6,0xF0,0xA9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xB7,0xB8,0x81,0x85,0x82,0x87,0x8D,0xF6,0xB9,0xF0,0x8D,0x90,0xFA,0xF3,0x00,
	0xFF,0xE1,0xE0,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,
	0xFF,0xE1,0xE0,0x8D,0x85,0xBA,0xBB,0xBC,0xBD,0x9E,0xBE,0x81,0x00,0x00,0x00,0x00,
	0xFF,0xE1,0xE0,0xBC,0xBF,0xC0,0x97,0xCE,0xBA,0xC1,0xC2,0xC3,0x00,0x00,0x00,0x00,
	0xFF,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xFF,0xFF,0xC7,0xC8,0xC7,0xC8,0xE1,
	0xFF,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xC7,0xC8,0xFF,0xFF,0xC8,0xC7,0xE1,
	0xFF,0xE0,0xC4,0xC6,0xE1,0xB7,0xB8,0xCA,0xCC,0xE0,0xCD,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xE0,0xE0,0xE0,0xE0,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xE0,0xE0,0xE0,0xFF,0xE0,
	0xFF,0xFF,0xE0,0xE0,0xE0,0xE0,0xE0,0xE0,0xFF,0xFF,0xFF,0xFF,0xFF,0xE0,0xE0,0xFF,
	0xFF,0xFF,0xFF,0xE0,0xE0,0xFF,0xE0,0xE0,0xE0,0xE0,0xFF,0xE0,0xFF,0xFF,0xFF,0xE0,
	0xFF,0xFF,0xFF,0xFF,0xE0,0xFF,0xFF,0xE0,0xFF,0xE0,0xE0,0xFF,0xE0,0xE0,0xE0,0xE0,
	0xFF,0xF1,0xF0,0xA5,0xA5,0xA5,0xA5,0xA5,0xB0,0xA5,0xB0,0xA5,0xA5,0xA5,0x8A,0xF1,
	0xFF,0xF1,0xF1,0xF0,0xF0,0xF0,0xF0,0xF0,0xB0,0xF0,0xB0,0xF0,0xF0,0xF0,0x8A,0xF0,
	0xFF,0xFF,0xEA,0xEB,0xEC,0xED,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xF1,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x81,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xD2,0xE5,0xE6,0xE3,0xE4,0xDE,0xDF,0xE2,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0x00,0x00,0x00,0x00,0x00,0x00,0x00,

	/* PL1-5.BIN : Foreground prom */
	0xFF,0x01,0x02,0x03,0x03,0x01,0x02,0xFF,0xFF,0x01,0x03,0x02,0x03,0x01,0xFF,0xFF,
	0x02,0x01,0x00,0x03,0x02,0x01,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x01,0x02,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7F,0x01,0x05,0x5C,
	0xFF,0x01,0x10,0x11,0x10,0x01,0xFF,0x15,0x11,0x01,0xFF,0xFF,0xFF,0x00,0x11,0x00,
	0x05,0x00,0x15,0x00,0x15,0x00,0x16,0x00,0x16,0x00,0x17,0x00,0x15,0x00,0x05,0x00,
	0x15,0x00,0x17,0x00,0x17,0x00,0x15,0x00,0x16,0x00,0x15,0x00,0x17,0x00,0x16,0x00,
	0x10,0x01,0x15,0xFF,0x15,0x01,0x10,0x00,0x18,0x01,0x15,0xFF,0x15,0x01,0x18,0x00,
	0xFF,0x01,0x10,0x11,0x10,0x01,0xFF,0x15,0x11,0x01,0xFF,0x00,0xFF,0x00,0x11,0x00,
	0x05,0x00,0x15,0x00,0x15,0x00,0x16,0x00,0x16,0x00,0x17,0x00,0x15,0x00,0x05,0x00,
	0x15,0x00,0x17,0x00,0x17,0x00,0x15,0x00,0x16,0x00,0x15,0x00,0x17,0x00,0x16,0x00,
	0x10,0x01,0x15,0xFF,0x15,0x01,0x10,0x00,0x18,0x01,0x15,0xFF,0x15,0x01,0x18,0x00,
	0x05,0x01,0x04,0x15,0xFF,0x01,0x04,0x15,0x10,0x01,0x04,0x15,0x00,0x00,0x00,0x00,
	0xFF,0x55,0x05,0x5F,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x4A,0x4C,0x49,0x00,0xFF,0x48,0x49,0x4C,0x4B,0x48,0x49,0x4C,0xFF,0x48,0x4B,0x49,
	0x4B,0x4C,0x49,0x4A,0xFF,0x43,0x3F,0x44,0x3F,0x43,0xFF,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x55,0x5C,0x00,0x59,0x55,0x5C,0x00,0xFF,0x55,0x59,0x00,0xFF,0x54,0x53,0x00,
	0xFF,0x55,0x5A,0x00,0x5A,0x55,0x59,0xFF,0xFF,0x55,0x05,0x5F,0x05,0x55,0x5F,0x00,
	0x7F,0x5E,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7F,0x01,0x5C,0x00,
	0xFF,0x21,0x00,0x00,0xFF,0x21,0x2C,0x00,0xFF,0x21,0x2A,0x29,0x2A,0x21,0x2C,0x00,
	0x29,0x21,0x2C,0x00,0x2C,0x21,0x29,0x00,0x2B,0x21,0x29,0x00,0xFF,0x21,0x29,0x00,
	0x2B,0x21,0x2D,0x2E,0x2B,0x21,0x2F,0x6B,0x6A,0x21,0x2B,0x29,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x21,0x62,0x00,0xFF,0x21,0x61,0x00,0x61,0x21,0x26,0x00,0x26,0x21,0x69,0x00,
	0x64,0x21,0x62,0x61,0x64,0x21,0x63,0x00,0x2E,0x21,0x2D,0x00,0x61,0x21,0x64,0x00,
	0x61,0x21,0x62,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x65,0x21,0x2D,0x00,0x67,0x21,0x68,0x00,0x67,0x21,0x66,0x00,0x66,0x21,0x2D,0x00,
	0xFF,0x21,0x6C,0x00,0x6C,0x21,0x6C,0x00,0x6C,0x21,0x6D,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x21,0x69,0x00,0x69,0x21,0x2B,0x00,0x6A,0x21,0x69,0x00,0x69,0x21,0x6A,0xFF,
	0x2B,0x21,0x2D,0x00,0x2C,0x21,0x6A,0x00,0x69,0x21,0x2C,0xFF,0x6A,0x21,0xFF,0x00,
	0xFF,0x21,0x2C,0x00,0x2D,0x21,0x2D,0x00,0x6A,0x21,0x2B,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x05,0x15,0x00,0x00,0x16,0x17,0x00,0x00,0x15,0x16,0x00,0x00,0x15,0x17,0x00,0x00,
	0x16,0x15,0x00,0x00,0x16,0x17,0x00,0x00,0x17,0x15,0x00,0x00,0x17,0x16,0x00,0x00,
	0x14,0x17,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x10,0x01,0x15,0x18,0x10,0x01,0x16,0x00,0x10,0x01,0x17,0x00,0x15,0x01,0x10,0x00,
	0x16,0x01,0x10,0x00,0x17,0x01,0x10,0x00,0x15,0x01,0xFF,0x00,0x16,0x01,0xFF,0x00,
	0x10,0x01,0x17,0x15,0x18,0x01,0x17,0x00,0x18,0x01,0x15,0x00,0x18,0x01,0x16,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x01,0x00,0x00,0xFF,0x6B,0x00,0x00,0xFF,0x71,0x00,0x00,0xFF,0x18,0x00,0x00,
	0xFF,0x70,0x00,0x00,0xFF,0x74,0x00,0x00,0xFF,0x78,0x00,0x00,0xFF,0x6B,0x18,0x00,
	0xFF,0x6B,0x25,0x00,0xFF,0x3C,0x3C,0x00,0xFF,0x26,0x18,0x00,0xFF,0x26,0x3C,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x01,0x00,0x00,0xFF,0x6B,0x00,0x00,0xFF,0x71,0x00,0x00,0xFF,0x18,0x00,0x00,
	0xFF,0x70,0x00,0x00,0xFF,0x74,0x00,0x00,0xFF,0x78,0x00,0x00,0xFF,0x6B,0x18,0x00,
	0xFF,0x6B,0x25,0x00,0xFF,0x3C,0x3C,0x00,0xFF,0x26,0x18,0x00,0xFF,0x26,0x3C,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x01,0x00,0x00,0xFF,0x6B,0x00,0x00,0xFF,0x71,0x00,0x00,0xFF,0x18,0x00,0x00,
	0xFF,0x70,0x00,0x00,0xFF,0x74,0x00,0x00,0xFF,0x78,0x00,0x00,0xFF,0x6B,0x18,0x00,
	0xFF,0x6B,0x25,0x00,0xFF,0x3C,0x3C,0x00,0xFF,0x26,0x18,0x00,0xFF,0x26,0x3C,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x42,

	/* PL1-4.BIN : Background prom */
	0x00,0x00,0x00,0x00,0x04,0x01,0x08,0x09,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x07,0x00,0x00,0x00,0x04,0x01,0x0A,0x09,
	0x06,0x01,0x08,0x07,0x04,0x01,0x08,0x07,0x0A,0x01,0x0A,0x06,0x06,0x01,0x0A,0x09,
	0x0A,0x01,0x08,0x06,0x08,0x01,0x08,0x09,0x04,0x01,0x05,0x00,0x05,0x01,0x04,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x06,0x01,0x12,0x13,0x12,0x01,0x06,0x00,0x05,0x01,0x00,0x00,0x06,0x01,0x13,0x00,
	0x00,0x01,0x18,0x00,0x10,0x01,0x18,0x19,0x18,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x06,0x01,0x05,0x3D,0x0C,0x0D,0x0E,0x0B,0x0B,0x01,0x0B,0x33,0x06,0x01,0x1A,0x00,
	0x1A,0x01,0x3C,0x3C,0x00,0x1E,0x05,0x00,0x0B,0x1B,0x1C,0x1D,0x0B,0x1B,0x1C,0x01,
	0x35,0x01,0x0C,0x00,0x36,0x01,0x0C,0x00,0x56,0x01,0x00,0x00,0x05,0x01,0x1A,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x77,0x71,0x76,0x78,0x77,0x74,0x73,0x70,0x77,0x70,0x72,0x73,0x77,0x74,0x75,0x00,
	0x77,0x75,0x73,0x72,0x1A,0x71,0x76,0x78,0x1A,0x75,0x76,0x78,0x1A,0x70,0x76,0x78,
	0x1A,0x75,0x71,0x76,0x77,0x75,0x76,0x78,0x77,0x74,0x75,0x78,0x77,0x71,0x76,0x78,
	0x77,0x7B,0x73,0x76,0x77,0x71,0x73,0x76,0x77,0x75,0x74,0x00,0x1A,0x75,0x74,0x00,
	0x77,0x75,0x76,0x74,0x77,0x75,0x76,0x78,0x77,0x75,0x76,0x72,0x77,0x76,0x78,0x72,
	0x1A,0x75,0x76,0x74,0x1A,0x75,0x76,0x78,0x1A,0x71,0x76,0x78,0x1A,0x70,0x72,0x73,
	0x1A,0x75,0x73,0x72,0x1A,0x75,0x76,0x74,0x1A,0x75,0x76,0x78,0x79,0x77,0x7A,0x00,
	0x77,0x75,0x71,0x76,0x7A,0x21,0x77,0x00,0x77,0x70,0x76,0x78,0x00,0x00,0x00,0x00,
	0x00,0x31,0x3A,0x00,0x34,0x31,0x05,0x00,0x06,0x31,0x35,0x00,0x06,0x31,0x34,0x00,
	0x34,0x31,0x06,0x00,0x06,0x31,0x36,0x00,0x06,0x31,0x06,0x00,0x00,0x00,0x00,0x00,
	0x05,0x43,0x45,0x00,0x00,0x43,0x3E,0x42,0x42,0x43,0x05,0x00,0x06,0x43,0x37,0x42,
	0x00,0x43,0x40,0x42,0x06,0x43,0x3D,0x00,0x06,0x47,0x3C,0x00,0x00,0x43,0x06,0x42,
	0x3C,0x43,0x37,0x00,0x3C,0x47,0x06,0x00,0x00,0x00,0x00,0x00,0x05,0x55,0x5B,0x00,
	0x06,0x55,0x56,0x00,0x57,0x55,0x56,0x58,0x06,0x55,0x57,0x00,0x06,0x52,0x51,0x00,
	0x58,0x55,0x57,0x06,0x06,0x55,0x58,0x00,0x5D,0x5E,0x00,0x00,0x58,0x55,0x56,0x00,
	0x00,0x00,0x00,0x00,0x06,0x21,0x23,0x00,0x24,0x21,0x23,0x00,0x06,0x21,0x24,0x23,
	0x06,0x21,0x23,0x00,0x27,0x21,0x23,0x28,0x28,0x21,0x23,0x00,0x23,0x21,0x23,0x00,
	0x06,0x21,0x6A,0x00,0x6A,0x21,0x28,0x25,0x25,0x21,0x6A,0x00,0x25,0x21,0x28,0x00,
	0x27,0x21,0x28,0x00,0x06,0x21,0x25,0x00,0x2D,0x21,0x25,0x00,0x2D,0x21,0x6A,0x00,
	0x6A,0x21,0x25,0x00,0x06,0x21,0x28,0x00,0x06,0x21,0x6B,0x00,0x00,0x00,0x00,0x00,
	0x05,0x01,0x07,0x09,0x06,0x01,0x07,0x09,0x06,0x01,0x34,0x25,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x06,0x01,0x56,0x00,
	0x27,0x21,0x06,0x5C,0x25,0x21,0x27,0x00,0x06,0x21,0x25,0x00,0x65,0x21,0x00,0x00,
	0x27,0x21,0x5A,0x5C,0x05,0x21,0x00,0x00,0x27,0x21,0x41,0x2D,0x51,0x21,0x00,0x00,
	0x66,0x21,0x27,0x6B,0x25,0x21,0x5C,0x5A,0x25,0x21,0x5A,0x06,0x5A,0x21,0x06,0x5C,
	0x5C,0x21,0x06,0x05,0x5C,0x21,0x5A,0x00,0x06,0x21,0x5A,0x00,0x25,0x21,0x05,0x5C,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x05,0x43,0x45,0x3D,0x40,0x43,0x45,0x33,0x37,0x43,0x45,0x06,0x06,0x43,0x00,0x00,
	0x06,0x43,0x45,0x3C,0x05,0x01,0x04,0x00,0x00,0x43,0x45,0x3E,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x37,0x43,0x45,0x3C,0x3C,0x43,0x45,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0xFF,0x6B,0x00,0x00,0xFF,0x61,0x00,0x00,0xFF,0x3C,0x00,0x00,0xFF,0x18,0x00,0x00,
	0xFF,0x25,0x00,0x00,0xFF,0x26,0x00,0x00,0xFF,0x29,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x6E,0x4D,0x00,0x00,0x7D,0x6F,0x4D,0x00,0x7D,0x6F,0x4E,0x7E,0x7D,0x6F,0x6E,
	0x00,0x7D,0x6F,0x7E,0x00,0x7E,0x00,0x00,0x00,0x6F,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x7C,0x00,0x7C,
};



static struct MachineDriver machine_driver =
{
	/* basic machine hardware */
	{
		{
			CPU_M6809,
			2000000,	/* 2.000 Mhz (?) */
			0,
			readmem,writemem,0,0,
			interrupt,1
		},
		{
			CPU_HD63701,	/* or compatible 6808 with extra instructions */
			2000000,	/* 2.000 Mhz (?) */
			2,
			mcu_readmem,mcu_writemem,0,0,
			interrupt,1
		},
	},
	60,DEFAULT_REAL_60HZ_VBLANK_DURATION,
	100,	/* we need heavy synching between the MCU and the CPU */
	pacland_init_machine,

	/* video hardware */
	42*8, 32*8, { 3*8, 39*8-1, 2*8, 30*8-1 },
	gfxdecodeinfo,
	256,256*4+256*4+2*64*16,
	pacland_vh_convert_color_prom,

	VIDEO_TYPE_RASTER|VIDEO_MODIFIES_PALETTE,
	0,
	pacland_vh_start,
	pacland_vh_stop,
	pacland_vh_screenrefresh,

	/* sound hardware */
	0,0,0,0
};



/***************************************************************************

  Game driver(s)

***************************************************************************/

ROM_START( pacland_rom )
	ROM_REGION(0x20000)	/* 128k for code */
	ROM_LOAD( "PL1-1",  0x08000, 0x4000, 0x1d7d3475 )
	ROM_LOAD( "PL1-2",  0x0C000, 0x4000, 0x1a30a372 )
	/* all the following are banked at 0x4000-0x5fff */
	ROM_LOAD( "PL1-3",  0x10000, 0x4000, 0x8ef26c48 )
	ROM_LOAD( "PL1-4",  0x14000, 0x4000, 0xef74074a )
	ROM_LOAD( "PL1-5",  0x18000, 0x4000, 0x25793897 )
	ROM_LOAD( "PL1-6",  0x1c000, 0x4000, 0x2ef401f4 )

	ROM_REGION(0x20000)	/* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "PL1-12", 0x00000, 0x2000, 0x802ca90a )	/* chars */
	ROM_LOAD( "PL1-13", 0x02000, 0x2000, 0xcfc9c4bf )
	ROM_LOAD( "PL1-9",  0x10000, 0x4000, 0xa3d7bce5 )	/* sprites */
	ROM_LOAD( "PL1-10", 0x14000, 0x4000, 0xb17bd389 )
	ROM_LOAD( "PL1-8",  0x18000, 0x4000, 0xd1bc0d84 )
	ROM_LOAD( "PL1-11", 0x1c000, 0x4000, 0xbce2ba90 )

	ROM_REGION(0x10000)	/* 64k for code */
	ROM_LOAD( "PL1-7",       0x8000, 0x2000, 0xc9845a56 ) /* sub program for the mcu */
	ROM_LOAD( "PL1-MCU.BIN", 0xf000, 0x1000, 0x03a8c424 ) /* microcontroller */
ROM_END

ROM_START( paclandn_rom )
	ROM_REGION(0x20000)	/* 128k for code */
	ROM_LOAD( "PL1-1",  0x08000, 0x4000, 0x5b6b6889 )
	ROM_LOAD( "PL1-2",  0x0C000, 0x4000, 0xd84eb166 )
	/* all the following are banked at 0x4000-0x5fff */
	ROM_LOAD( "PL1-3",  0x10000, 0x4000, 0x8ef26c48 )
	ROM_LOAD( "PL1-4",  0x14000, 0x4000, 0xef74074a )
	ROM_LOAD( "PL1-5",  0x18000, 0x4000, 0x25793897 )
	ROM_LOAD( "PL1-6",  0x1c000, 0x4000, 0x2ef401f4 )

	ROM_REGION(0x20000)	/* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "PL1-12", 0x00000, 0x2000, 0x802ca90a )	/* chars */
	ROM_LOAD( "PL1-13", 0x02000, 0x2000, 0xcfc9c4bf )
	ROM_LOAD( "PL1-9",  0x10000, 0x4000, 0xb7c9b1ef )	/* sprites */
	ROM_LOAD( "PL1-10", 0x14000, 0x4000, 0x9777eff7 )
	ROM_LOAD( "PL1-8",  0x18000, 0x4000, 0xe1983c42 )
	ROM_LOAD( "PL1-11", 0x1c000, 0x4000, 0x8bdb3de1 )

	ROM_REGION(0x10000)	/* 64k for code */
	ROM_LOAD( "PL1-7",       0x8000, 0x2000, 0xc9845a56 ) /* sub program for the mcu */
	ROM_LOAD( "PL1-MCU.BIN", 0xf000, 0x1000, 0x03a8c424 ) /* microcontroller */
ROM_END

ROM_START( paclanda_rom )
	ROM_REGION(0x20000)	/* 128k for code */
	ROM_LOAD( "PL6_01.BIN",  0x08000, 0x4000, 0xd9d023da )
	ROM_LOAD( "PL6_02.BIN",  0x0C000, 0x4000, 0xc6801902 )
	/* all the following are banked at 0x4000-0x5fff */
	ROM_LOAD( "PL1_03.BIN",  0x10000, 0x4000, 0x8ef26c48 )
	ROM_LOAD( "PL1_04.BIN",  0x14000, 0x4000, 0xef74074a )
	ROM_LOAD( "PL1_05.BIN",  0x18000, 0x4000, 0x25793897 )
	ROM_LOAD( "PL1_06.BIN",  0x1c000, 0x4000, 0x2ef401f4 )

	ROM_REGION(0x20000)	/* temporary space for graphics (disposed after conversion) */
	ROM_LOAD( "PL0_12.BIN",  0x00000, 0x2000, 0x01bda807 )	/* chars */
	ROM_LOAD( "PL1_13.BIN",  0x02000, 0x2000, 0xcfc9c4bf )
	ROM_LOAD( "PL1_09B.BIN", 0x10000, 0x4000, 0xb7c9b1ef )	/* sprites */
	ROM_LOAD( "PL1_10B.BIN", 0x14000, 0x4000, 0xe1ef547f )
	ROM_LOAD( "PL1_08.BIN",  0x18000, 0x4000, 0xe1983c42 )
	ROM_LOAD( "PL1_11.BIN",  0x1c000, 0x4000, 0x8bdb3de1 )

	ROM_REGION(0x10000)	/* 64k for code */
	ROM_LOAD( "PL1_07.BIN",  0x8000, 0x2000, 0xc9845a56 ) /* sub program for the mcu */
	ROM_LOAD( "PL1-MCU.BIN", 0xf000, 0x1000, 0x03a8c424 ) /* microcontroller */
ROM_END



struct GameDriver pacland_driver =
{
	"Pac-Land (Midway)",
	"pacland",
	"Ernesto Corvi\nMirko Buffoni\nNicola Salmoria",
	&machine_driver,

	pacland_rom,
	0, 0,
	0,
	0,	/* sound_prom */

	input_ports,

	color_prom, 0, 0,
	ORIENTATION_DEFAULT,

	0, 0
};

struct GameDriver paclandn_driver =
{
	"Pac-Land (Namco)",
	"paclandn",
	"Ernesto Corvi\nMirko Buffoni\nNicola Salmoria",
	&machine_driver,

	paclandn_rom,
	0, 0,
	0,
	0,	/* sound_prom */

	input_ports,

	color_prom, 0, 0,
	ORIENTATION_DEFAULT,

	0, 0
};

struct GameDriver paclanda_driver =
{
	"Pac-Land (Namco alternate)",
	"paclanda",
	"Ernesto Corvi\nMirko Buffoni\nNicola Salmoria",
	&machine_driver,

	paclanda_rom,
	0, 0,
	0,
	0,	/* sound_prom */

	input_ports,

	color_prom, 0, 0,
	ORIENTATION_DEFAULT,

	0, 0
};
